
// simple program that plays a sound and then writes a log file; 
// requires *both* the SdFat library and the WaveSD library in 
// your Arduino\libraries folder

// NOTES: 
// (1) you can't read/write other files while a sound file is playing
// (2) this example uses lots of PSTR constants to conserve RAM
// (3) there should be a sound file "test.wav" in the root folder of the SD
// (4) the program writes to a file "playlog.txt" in the root folder of the SD

#include "SdFat.h"
#include "SdFatUtil.h"
#include "WaveSD.h"

Sd2Card card;
SdVolume volume;
SdFile root;
SdFile logfile;
SdFile soundfile;
WaveSD wave;

// store error strings in flash to save RAM
#define error(s) error_P(PSTR(s))

void error_P(const char* str) {
  PgmPrint("error: ");
  SerialPrintln_P(str);
  if (card.errorCode()) {
    PgmPrint("SD error: ");
    Serial.print(card.errorCode(), HEX);
    Serial.print(',');
    Serial.println(card.errorData(), HEX);
  }
  while(1);
}

void setup(void) {
  Serial.begin(9600);
  
  PgmPrint("Free RAM: ");
  Serial.println(FreeRam());
    
  PgmPrintln("Type any character to start");
  while (!Serial.available());
    
  // initialize the SD card at SPI_HALF_SPEED to avoid bus errors with
  // breadboards.  use SPI_FULL_SPEED for better performance.
  if (!card.init()) error("card.init failed");

  // initialize a FAT volume
  if (!volume.init(&card)) error("volume.init failed");

  // open the root directory
  if (!root.openRoot(&volume)) error("openRoot failed");
  
  bool bPlayedSound = false;
  
  // try to play sound file
  if (soundfile.open(&root, "test.wav", O_READ) && wave.create(soundfile))
  {
    wave.play();
    while(wave.isplaying)
      delay(250); 
    bPlayedSound = true;
    soundfile.close();
  }
  
  // log what happened
  if (!logfile.open(&root, "playlog.txt", O_WRONLY | O_APPEND | O_CREAT)) 
    error("can't open log file");
  
  if (bPlayedSound)
    logfile.writeln_P(PSTR("played sound file"));
  else
    logfile.writeln_P(PSTR(">> sound file not found"));
  
  // IMPORTANT: close log file to flush any unwritten changes
  logfile.close();
}

void loop(void) 
{
}

